@@ -3,10 +3,12 @@ pub struct Ancestors<Find, Predicate, StateMut> {
     find: Find,
     predicate: Predicate,
     state: StateMut,
-    mode: Parents,
+    parents: Parents,
+    sorting: Sorting,
 }
 
 /// Specify how to handle commit parents during traversal.
+#[derive(Copy, Clone)]
 pub enum Parents {
     /// Traverse all parents, useful for traversing the entire ancestry.
     All,
@@ -20,6 +22,23 @@ impl Default for Parents {
     }
 }
 
+/// Specify how to sort commits during traversal.
+#[derive(Copy, Clone)]
+pub enum Sorting {
+    /// Commits are sorted as they are mentioned in the commit graph.
+    Topological,
+    /// Order commit looking up the commit date of the most recent parents.
+    ///
+    /// Note that since only parents are looked up this ordering is partial.
+    ByCommitterDate,
+}
+
+impl Default for Sorting {
+    fn default() -> Self {
+        Sorting::Topological
+    }
+}
+
 ///
 pub mod ancestors {
     use std::{
@@ -31,7 +50,7 @@ pub mod ancestors {
     use git_object::CommitRefIter;
     use quick_error::quick_error;
 
-    use crate::commit::{Ancestors, Parents};
+    use crate::commit::{Ancestors, Parents, Sorting};
 
     quick_error! {
         /// The error is part of the item returned by the [Ancestors] iterator.
@@ -55,6 +74,8 @@ pub mod ancestors {
         next: VecDeque<ObjectId>,
         buf: Vec<u8>,
         seen: BTreeSet<ObjectId>,
+        parents_with_date: Vec<(ObjectId, u32)>,
+        parents_buf: Vec<u8>,
     }
 
     impl State {
@@ -67,8 +88,14 @@ pub mod ancestors {
 
     impl<Find, Predicate, StateMut> Ancestors<Find, Predicate, StateMut> {
         /// Change our commit parent handling mode to the given one.
-        pub fn mode(mut self, mode: Parents) -> Self {
-            self.mode = mode;
+        pub fn parents(mut self, mode: Parents) -> Self {
+            self.parents = mode;
+            self
+        }
+
+        /// Set the sorting method, either topological or by author date
+        pub fn sorting(mut self, sorting: Sorting) -> Self {
+            self.sorting = sorting;
             self
         }
     }
@@ -137,7 +164,8 @@ pub mod ancestors {
                 find,
                 predicate,
                 state,
-                mode: Default::default(),
+                parents: Default::default(),
+                sorting: Default::default(),
             }
         }
     }
@@ -151,6 +179,83 @@ pub mod ancestors {
         type Item = Result<ObjectId, Error>;
 
         fn next(&mut self) -> Option<Self::Item> {
+            if matches!(self.parents, Parents::First) {
+                self.next_by_topology()
+            } else {
+                match self.sorting {
+                    Sorting::Topological => self.next_by_topology(),
+                    Sorting::ByCommitterDate => self.next_by_commit_date(),
+                }
+            }
+        }
+    }
+
+    impl<Find, Predicate, StateMut> Ancestors<Find, Predicate, StateMut>
+    where
+        Find: for<'a> FnMut(&oid, &'a mut Vec<u8>) -> Option<CommitRefIter<'a>>,
+        Predicate: FnMut(&oid) -> bool,
+        StateMut: BorrowMut<State>,
+    {
+        fn next_by_commit_date(&mut self) -> Option<Result<ObjectId, Error>> {
+            let state = self.state.borrow_mut();
+            state.parents_with_date.clear();
+            let res = state.next.pop_front();
+
+            if let Some(oid) = res {
+                match (self.find)(&oid, &mut state.buf) {
+                    Some(mut commit_iter) => {
+                        if let Some(Err(decode_tree_err)) = commit_iter.next() {
+                            return Some(Err(decode_tree_err.into()));
+                        }
+
+                        for token in commit_iter {
+                            match token {
+                                Ok(git_object::commit::ref_iter::Token::Parent { id }) => {
+                                    let parent = (self.find)(id.as_ref(), &mut state.parents_buf);
+
+                                    let parent_committer_date = parent
+                                        .and_then(|mut parent| parent.committer().map(|committer| committer.time));
+
+                                    if let Some(parent_committer_date) = parent_committer_date {
+                                        state.parents_with_date.push((id, parent_committer_date.time));
+                                    }
+
+                                    if matches!(self.parents, Parents::First) {
+                                        break;
+                                    }
+                                }
+                                Ok(_unused_token) => break,
+                                Err(err) => return Some(Err(err.into())),
+                            }
+                        }
+                    }
+                    None => return Some(Err(Error::NotFound { oid })),
+                }
+            }
+
+            state
+                .parents_with_date
+                .sort_by(|(_, time), (_, other_time)| time.cmp(other_time).reverse());
+            for parent in &state.parents_with_date {
+                let id = parent.0;
+                let was_inserted = state.seen.insert(id);
+
+                if was_inserted && (self.predicate)(&id) {
+                    state.next.push_back(id);
+                }
+            }
+
+            res.map(Ok)
+        }
+    }
+
+    impl<Find, Predicate, StateMut> Ancestors<Find, Predicate, StateMut>
+    where
+        Find: for<'a> FnMut(&oid, &'a mut Vec<u8>) -> Option<CommitRefIter<'a>>,
+        Predicate: FnMut(&oid) -> bool,
+        StateMut: BorrowMut<State>,
+    {
+        fn next_by_topology(&mut self) -> Option<Result<ObjectId, Error>> {
             let state = self.state.borrow_mut();
             let res = state.next.pop_front();
             if let Some(oid) = res {
@@ -166,7 +271,7 @@ pub mod ancestors {
                                     if was_inserted && (self.predicate)(&id) {
                                         state.next.push_back(id);
                                     }
-                                    if matches!(self.mode, Parents::First) {
+                                    if matches!(self.parents, Parents::First) {
                                         break;
                                     }
                                 }
